--[[
Name: AlarWidgets-3.0
Revision: $Rev: 426 $
Author: Alar of Daggerspine
Email: alar@aspide.it
Website: http://wow.aspide.it
Documentation: http://wiki.wowace.com/wiki/AlarLib-3.0
SVN: $HeadUrl:$
Description: Generic library
Dependencies: Ace3
License: LGPL v2.1
--]]
local me,ns=...
local _G=_G
local MAJOR_VERSION = "AlarWidgets-3.0"
local MINOR_VERSION = tonumber(string.sub("$Revision: 426 $", 12, -3))
local debug=_G.GetAlarDebug(me .. MAJOR_VERSION)
local print=_G.GetAlarPrint(me .. MAJOR_VERSION)
debug('Loading AlarWidgets inside',me) 
local __FILE__=tostring(debugstack(1,2,0):match("(.*):1"))
debug(MAJOR_VERSION .. " from file " .. __FILE__)
debug("Loading " .. MAJOR_VERSION .. " r" .. MINOR_VERSION )
if (not LibStub) then
    print("Couldn't find LibStub. Please reinstall " .. MAJOR_VERSION )
end
local lib,old=LibStub:NewLibrary(MAJOR_VERSION,MINOR_VERSION)
if (not lib) then
    debug("Already loaded a newer or equal version of " .. MAJOR_VERSION)
    return -- Already loaded
end
if (old) then
    debug(format("Upgrading %s from %s to %s",MAJOR_VERSION,old,MINOR_VERSION))
end
-----------------------------------------------------------------------------
lib.widgets=lib.widgets or {}
local AceGUI = LibStub("AceGUI-3.0")
lib.AceGUI=AceGUI
lib.pp=debug
local WidgetVersion=MINOR_VERSION-65432
function lib:IterateWidgets() return pairs(self.widgets) end
----------------
-- Main Frame --
----------------
do
	local proto={}
	function proto:OnAcquire()
		self:ApplyStatus()
	end	
	function proto:OnRelease()
		self.status = nil
		for k in pairs(self.localstatus) do
			self.localstatus[k] = nil
		end
	end	
	function proto:ApplyStatus(this)
		return
	end
	function proto:Status()
		if (self.obj) then
			return self.obj.status or self.obj.localstatus
		else
			return self.status or self.localstatus
		end
	end
	function proto:Show()
		local frame=self.frame
		if (frame.fade) then frame.fade.Stop() end
		frame:SetAlpha(1.0) 
		frame:Show()
	end
	function proto:Hide()
		local frame=self.frame
		if (frame.fade) then frame.fade.Stop() end
		frame:Hide()
		frame:SetAlpha(1.0) 
	end
	
	function proto:SetLocked(lock)
		local status = self:Status()
		local old=status.locked
		status.locked=lock
		self:Lock()
		return old
	end
	function proto:GetLocked()
		local status = self:Status()
		return status.locked
	end
	function proto:FadeIn(delay,from,to)
		local frame=self.frame
		delay=delay or 30
		from=from or frame:GetAlpha()
		to=to or 1
		local frame=self.frame
		if (frame:IsShown()) then
			frame:SetAlpha(to)
		else 
        	UIFrameFadeIn(frame,delay,from,to)
        end
	end
	function proto:FadeOut(delay,from,to)
		local frame=self.frame
		delay=delay or 30
		from=from or frame:GetAlpha()
		to=to or 0
		local frame=self.frame
			if (not frame:IsShown()) then
			frame:SetAlpha(to)
		else 
			UIFrameFadeOut(frame,delay,from,to)
		end
	end
	function proto:GetInfo()
		local frame=self.frame
		print(frame:GetWidth(),'x',frame:GetHeight(),frame:GetWidth())
		for i= 1,frame:GetNumPoints() do
			print (frame:GetPoint(i))
		end
	end
	function proto:ClearAllPoints()
		local frame=self.frame
		frame:ClearAllPoints()
	end
	function proto:SetPoint(...)
		local frame=self.frame
		frame:SetPoint(...)
	end
	function lib.InjectStandardMethods(target)
	    for k,v in pairs(proto) do
			target[k]=target[k] or v
	    end
	    if (not target.localstatus) then
	    	target.localstatus={}
	    end
	end
end
function lib:CreatePanel()
    AceGUI:Create('AlarPanel')
end
function lib:CreateConfig()
    AceGUI:Create('AlarConfig')
end
function lib:CreateMiniMap()
    AceGUI:Create('AlarMinimapButton')
end
function lib:CreateCastButton()
    AceGUI:Create('AlarCastButton')
end
lib.panels=lib.panels or {}
do 
	local panels=lib.panels
	
function lib:Pop(widget,offset)
	debug('Popping',widget.frame:GetName())
	offset=offset or 48
	local newoffset=0
	local lpanels=panels
	for i,panel in pairs(lpanels) do
		if (panel==widget) then return newoffset end
		if (type(panel.IsShown)=="function") then
			if (not panel:IsShown()) then
				tremove(panels,i)
			else
				newoffset=newoffset+offset
			end
		else
			tremove(panels,i)
		end
	end
	tinsert(panels,widget)
	return newoffset
end
end
------------------------- Helpers -----
